"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.verificationKeyForIndyDid = exports.indyDidFromNamespaceAndInitialKey = exports.isSelfCertifiedIndyDid = exports.didDocDiff = exports.combineDidDocumentWithJson = exports.parseIndyDid = exports.createKeyAgreementKey = exports.indyDidDocumentFromDid = void 0;
const core_1 = require("@aries-framework/core");
const did_1 = require("../utils/did");
// Create a base DIDDoc template according to https://hyperledger.github.io/indy-did-method/#base-diddoc-template
function indyDidDocumentFromDid(did, verKeyBase58) {
    const verificationMethodId = `${did}#verkey`;
    const publicKeyBase58 = verKeyBase58;
    const builder = new core_1.DidDocumentBuilder(did)
        .addContext('https://w3id.org/security/suites/ed25519-2018/v1')
        .addVerificationMethod({
        controller: did,
        id: verificationMethodId,
        publicKeyBase58,
        type: 'Ed25519VerificationKey2018',
    })
        .addAuthentication(verificationMethodId);
    return builder;
}
exports.indyDidDocumentFromDid = indyDidDocumentFromDid;
function createKeyAgreementKey(verkey) {
    return core_1.TypedArrayEncoder.toBase58((0, core_1.convertPublicKeyToX25519)(core_1.TypedArrayEncoder.fromBase58(verkey)));
}
exports.createKeyAgreementKey = createKeyAgreementKey;
function parseIndyDid(did) {
    const match = did.match(did_1.DID_INDY_REGEX);
    if (match) {
        const [, namespace, namespaceIdentifier] = match;
        return { namespace, namespaceIdentifier };
    }
    else {
        throw new core_1.AriesFrameworkError(`${did} is not a valid did:indy did`);
    }
}
exports.parseIndyDid = parseIndyDid;
const deepMerge = (a, b) => {
    const output = {};
    [...new Set([...Object.keys(a), ...Object.keys(b)])].forEach((key) => {
        // Only an object includes a given key: just output it
        if (a[key] && !b[key]) {
            output[key] = a[key];
        }
        else if (!a[key] && b[key]) {
            output[key] = b[key];
        }
        else {
            // Both objects do include the key
            // Some or both are arrays
            if (Array.isArray(a[key])) {
                if (Array.isArray(b[key])) {
                    const element = new Set();
                    a[key].forEach((item) => element.add(item));
                    b[key].forEach((item) => element.add(item));
                    output[key] = Array.from(element);
                }
                else {
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any
                    const arr = a[key];
                    output[key] = Array.from(new Set(...arr, b[key]));
                }
            }
            else if (Array.isArray(b[key])) {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                const arr = b[key];
                output[key] = Array.from(new Set(...arr, a[key]));
                // Both elements are objects: recursive merge
            }
            else if (typeof a[key] == 'object' && typeof b[key] == 'object') {
                output[key] = deepMerge(a, b);
            }
        }
    });
    return output;
};
/**
 * Combine a JSON content with the contents of a DidDocument
 * @param didDoc object containing original DIDDocument
 * @param json object containing extra DIDDoc contents
 *
 * @returns a DidDocument object resulting from the combination of both
 */
function combineDidDocumentWithJson(didDoc, json) {
    const didDocJson = didDoc.toJSON();
    const combinedJson = deepMerge(didDocJson, json);
    return core_1.JsonTransformer.fromJSON(combinedJson, core_1.DidDocument);
}
exports.combineDidDocumentWithJson = combineDidDocumentWithJson;
/**
 * Processes the difference between a base DidDocument and a complete DidDocument
 *
 * Note: it does deep comparison based only on "id" field to determine whether is
 * the same object or is a different one
 *
 * @param extra complete DidDocument
 * @param base base DidDocument
 * @returns diff object
 */
function didDocDiff(extra, base) {
    const output = {};
    for (const key in extra) {
        if (!(key in base)) {
            output[key] = extra[key];
        }
        else {
            // They are arrays: compare elements
            if (Array.isArray(extra[key]) && Array.isArray(base[key])) {
                // Different types: return the extra
                output[key] = [];
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                const baseAsArray = base[key];
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                const extraAsArray = extra[key];
                for (const element of extraAsArray) {
                    if (!baseAsArray.find((item) => item.id === element.id)) {
                        ;
                        output[key].push(element);
                    }
                }
            } // They are both objects: do recursive diff
            else if (typeof extra[key] == 'object' && typeof base[key] == 'object') {
                output[key] = didDocDiff(extra[key], base[key]);
            }
            else {
                output[key] = extra[key];
            }
        }
    }
    return output;
}
exports.didDocDiff = didDocDiff;
/**
 * Check whether the did is a self certifying did. If the verkey is abbreviated this method
 * will always return true. Make sure that the verkey you pass in this method belongs to the
 * did passed in
 *
 * @return Boolean indicating whether the did is self certifying
 */
function isSelfCertifiedIndyDid(did, verkey) {
    const { namespace } = parseIndyDid(did);
    const { did: didFromVerkey } = indyDidFromNamespaceAndInitialKey(namespace, core_1.Key.fromPublicKeyBase58(verkey, core_1.KeyType.Ed25519));
    if (didFromVerkey === did) {
        return true;
    }
    return false;
}
exports.isSelfCertifiedIndyDid = isSelfCertifiedIndyDid;
function indyDidFromNamespaceAndInitialKey(namespace, initialKey) {
    const buffer = core_1.Hasher.hash(initialKey.publicKey, 'sha2-256');
    const id = core_1.TypedArrayEncoder.toBase58(buffer.slice(0, 16));
    const verkey = initialKey.publicKeyBase58;
    const did = `did:indy:${namespace}:${id}`;
    return { did, id, verkey };
}
exports.indyDidFromNamespaceAndInitialKey = indyDidFromNamespaceAndInitialKey;
/**
 * Fetches the verification key for a given did:indy did and returns the key as a {@link Key} object.
 *
 * @throws {@link AriesFrameworkError} if the did could not be resolved or the key could not be extracted
 */
async function verificationKeyForIndyDid(agentContext, did) {
    // FIXME: we should store the didDocument in the DidRecord so we don't have to fetch our own did
    // from the ledger to know which key is associated with the did
    const didsApi = agentContext.dependencyManager.resolve(core_1.DidsApi);
    const didResult = await didsApi.resolve(did);
    if (!didResult.didDocument) {
        throw new core_1.AriesFrameworkError(`Could not resolve did ${did}. ${didResult.didResolutionMetadata.error} ${didResult.didResolutionMetadata.message}`);
    }
    // did:indy dids MUST have a verificationMethod with #verkey
    const verificationMethod = didResult.didDocument.dereferenceKey(`${did}#verkey`);
    const key = (0, core_1.getKeyFromVerificationMethod)(verificationMethod);
    return key;
}
exports.verificationKeyForIndyDid = verificationKeyForIndyDid;
//# sourceMappingURL=didIndyUtil.js.map